<?php
// Own namespace
namespace CoreFramework\Database\Backend;

// Import framework stuff
use CoreFramework\Database\Backend\DatabaseBackend;
use CoreFramework\Generic\FrameworkException;
use CoreFramework\Object\BaseFrameworkSystem;

/**
 * An abstract database access class for handling database I/O requests
 *
 * @see			DatabaseBackend - An interface for database backends
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2017 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
abstract class BaseDatabaseBackend extends BaseFrameworkSystem implements DatabaseBackend {
	// Constants for exceptions
	const EXCEPTION_SQL_QUERY = 0x140;

	// Result array indexes
	const RESULT_INDEX_ROWS      = 'rows';
	const RESULT_INDEX_STATUS    = 'status';
	const RESULT_INDEX_EXCEPTION = 'exception';

	// Constants for MySQL backward-compatiblity (PLEASE FIX THEM!)
	const DB_CODE_TABLE_MISSING     = 0x100;
	const DB_CODE_TABLE_UNWRITEABLE = 0x101;
	const DB_CODE_DATA_FILE_CORRUPT = 0x102;

	// Status results
	const RESULT_OKAY = 'ok';

	/**
	 * Last thrown exception or NULL if all is fine
	 */
	private $lastException = NULL;

	/**
	 * Protected constructor
	 *
	 * @param	$className	Name of the class
	 * @return	void
	 */
	protected function __construct ($className) {
		// Call parent constructor
		parent::__construct($className);
	}

	/**
	 * Getter for last exception
	 *
	 * @return	$lastException	Last thrown exception
	 */
	public final function getLastException () {
		return $this->lastException;
	}

	/**
	 * Setter for last exception
	 *
	 * @param	$lastException	Last thrown exception
	 * @return	void
	 */
	public final function setLastException (FrameworkException $exceptionInstance) {
		$this->lastException = $exceptionInstance;
	}

	/**
	 * Reset the last exception instance. This should be done after a "query"
	 * was completed without any errors.
	 *
	 * @return	void
	 */
	protected final function resetLastException () {
		$this->lastException = NULL;
	}

	/**
	 * Removes non-public data from given array.
	 *
	 * @param	$data	An array with possible non-public data that needs to be removed.
	 * @return	$data	A cleaned up array with only public data.
	 */
	public abstract function removeNonPublicDataFromArray (array $data);

}
